import pytest

from conftest import assert_bash_exec, bash_env_saved


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+declare -f fn$")
class TestUnitParseUsage:
    def test_1(self, bash):
        assert_bash_exec(bash, "fn() { echo; }")
        output = assert_bash_exec(bash, "_parse_usage fn; (($? == 1))")
        assert not output

    def test_2(self, bash):
        assert_bash_exec(bash, "fn() { echo 'no dashes here'; }")
        output = assert_bash_exec(bash, "_parse_usage fn; (($? == 1))")
        assert not output

    def test_3(self, bash):
        assert_bash_exec(bash, "fn() { echo 'foo [-f]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "-f".split()

    def test_4(self, bash):
        assert_bash_exec(bash, "fn() { echo 'bar [-aBcD] [-e X]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "-a -B -c -D -e".split()

    def test_5(self, bash):
        assert_bash_exec(bash, "fn() { echo '[-[XyZ]] [--long=arg]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "-X -y -Z --long=".split()

    def test_6(self, bash):
        assert_bash_exec(bash, "fn() { echo '[-s|--long]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "--long".split()

    def test_7(self, bash):
        assert_bash_exec(bash, "fn() { echo '[-s, --long=arg]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "--long=".split()

    def test_8(self, bash):
        assert_bash_exec(bash, "fn() { echo '[--long/-s] [-S/--longer]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "--long --longer".split()

    def test_9(self, bash):
        assert_bash_exec(bash, "fn() { echo '[ -a ] [ -b foo ]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "-a -b".split()

    def test_10(self, bash):
        assert_bash_exec(bash, "fn() { echo '[ -a | --aa ]'; }")
        output = assert_bash_exec(bash, "_parse_usage fn", want_output=True)
        assert output.split() == "--aa".split()

    def test_11(self, bash):
        assert_bash_exec(
            bash, "fn() { echo ----; echo ---foo; echo '----- bar'; }"
        )
        output = assert_bash_exec(bash, "_parse_usage fn; (($? == 1))")
        assert not output

    def test_12(self, bash):
        output = assert_bash_exec(
            bash, "echo '[-duh]' | _parse_usage -", want_output=True
        )
        assert output.split() == "-d -u -h".split()

    def test_custom_helpopt1(self, bash):
        assert_bash_exec(
            bash, "fn() { [[ $1 == -h ]] && echo 'fn [-option]'; true; }"
        )
        output = assert_bash_exec(bash, "_parse_usage fn -h", want_output=True)
        assert output.split() == "-o -p -t -i -o -n".split()

    def test_custom_helpopt2(self, bash):
        assert_bash_exec(
            bash, "fn() { [[ $1 == '-?' ]] && echo 'fn [-option]'; }"
        )
        output = assert_bash_exec(
            bash, "_parse_usage fn '-?'", want_output=True
        )
        assert output.split() == "-o -p -t -i -o -n".split()

    def test_custom_helpopt2_failglob(self, bash):
        assert_bash_exec(
            bash, "fn() { [[ $1 == '-?' ]] && echo 'fn [-option]'; }"
        )
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("failglob", True)
            output = assert_bash_exec(
                bash, "_parse_usage fn '-?'", want_output=True
            )
        assert output.split() == "-o -p -t -i -o -n".split()

    def test_custom_helpopt2_nullglob(self, bash):
        assert_bash_exec(
            bash, "fn() { [[ $1 == '-?' ]] && echo 'fn [-option]'; }"
        )
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("nullglob", True)
            output = assert_bash_exec(
                bash, "_parse_usage fn '-?'", want_output=True
            )
        assert output.split() == "-o -p -t -i -o -n".split()



import pytest


class TestGrep:
    @pytest.mark.complete("grep --", require_longopt=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("grep --no-complete-dir f", cwd="shared/default")
    def test_2(self, completion):
        """
        Test --no-*dir isn't restricted to dirs only.

        Not really a grep option, but tests _comp_complete_longopt.
        """
        assert completion == "foo foo.d/".split()

    @pytest.mark.complete("grep TZ ", cwd="shared/default")
    def test_no_variable_assignment_confusion(self, completion):
        """
        Test TZ doesn't trigger known variable value assignment completion.

        Not really a grep specific, but good to test somewhere.
        Refs https://github.com/scop/bash-completion/issues/457
        """
        assert "foo" in completion



import pytest


class TestFlake8:
    @pytest.mark.complete("flake8 ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "flake8 -", require_cmd=True, xfail="! flake8 --help &>/dev/null"
    )
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("flake8 --doesnt-exist=")
    def test_3(self, completion):
        assert not completion



import fnmatch
import os

import pytest


class TestUpgradepkg:
    @pytest.mark.complete("upgradepkg -")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("upgradepkg --")
    def test_2(self, completion):
        assert (
            completion
            == "--dry-run --install-new --reinstall --verbose".split()
        )

    @pytest.mark.complete("upgradepkg ", cwd="slackware/home")
    def test_4(self, completion):
        expected = sorted(
            [
                "%s/" % x
                for x in os.listdir("slackware/home")
                if os.path.isdir("./slackware/home/%s" % x)
            ]
            + [
                x
                for x in os.listdir("slackware/home")
                if os.path.isfile("./slackware/home/%s" % x)
                and fnmatch.fnmatch(x, "*.t[bglx]z")
            ]
        )
        assert completion == expected

    @pytest.mark.complete("upgradepkg foo%", cwd="slackware/home")
    def test_after_percent(self, completion):
        expected = sorted(
            [
                "%s/" % x
                for x in os.listdir("slackware/home")
                if os.path.isdir("./slackware/home/%s" % x)
            ]
            + [
                x
                for x in os.listdir("slackware/home")
                if os.path.isfile("./slackware/home/%s" % x)
                and fnmatch.fnmatch(x, "*.t[bglx]z")
            ]
        )
        assert completion == ["foo%%%s" % x for x in expected]



import pytest


class TestShellcheck:
    @pytest.mark.complete("shellcheck ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("shellcheck -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("shellcheck --format=", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("shellcheck -s ", require_cmd=True)
    def test_4(self, completion):
        assert "bash" in completion



import subprocess

import pytest


class TestModprobe:
    @pytest.mark.complete("modprobe --al")
    def test_1(self, completion):
        assert completion == "l"

    # "du": dummy, ...
    @pytest.mark.complete(
        "modprobe du",
        xfail="! ls /lib/modules/%s &>/dev/null"
        % subprocess.check_output(
            "uname -r 2>/dev/null || " "echo non-existent-kernel", shell=True
        )
        .decode()
        .strip(),
    )
    def test_2(self, completion):
        assert completion

    # "du": dummy, ...
    @pytest.mark.complete("modprobe -S non-existent-kernel du")
    def test_3(self, completion):
        assert not completion

    @pytest.mark.complete("modprobe non-existent-module ")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("modprobe /tm")
    def test_5(self, completion):
        assert completion
        assert not completion.endswith(" ")



import pytest


class TestLvmdiskscan:
    @pytest.mark.complete(
        "lvmdiskscan --",
        require_cmd=True,
        xfail="! lvmdiskscan --help &>/dev/null",
    )
    def test_1(self, completion):
        assert completion



import pytest


class TestLess:
    @pytest.mark.complete("less --", require_longopt=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("less --", require_longopt=True)
    def test_no_dashdashdash(self, completion):
        assert all(not x.startswith("---") for x in completion)



import pytest

from conftest import assert_bash_exec, bash_env_saved, is_bash_type


@pytest.mark.bashcomp(
    pre_cmds=("CLASSPATH=$PWD/java/a:$PWD/java/bashcomp.jar",)
)
class TestJava:
    @pytest.fixture(scope="class")
    def can_list_jar(self, bash):
        return (
            is_bash_type(bash, "zipinfo")
            or is_bash_type(bash, "unzip")
            or is_bash_type(bash, "jar")
        )

    @pytest.mark.complete("java -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("java ", cwd="java")
    def test_2(self, completion, can_list_jar):
        if can_list_jar:
            assert (
                completion
                == "JEP330.java a/ b bashcomp.jarred c. toplevel".split()
            )
        else:
            assert completion == "JEP330.java a/ b c.".split()

    @pytest.mark.complete("java -classpath bashcomp.jar ", cwd="java")
    def test_3(self, completion, can_list_jar):
        if can_list_jar:
            assert (
                completion == "JEP330.java a/ bashcomp.jarred toplevel".split()
            )
        else:
            assert completion == "JEP330.java a/".split()

    @pytest.mark.complete("java -cp bashcomp.jar:a/c ", cwd="java")
    def test_4(self, completion, can_list_jar):
        if can_list_jar:
            assert (
                completion
                == "JEP330.java a/ bashcomp.jarred d toplevel".split()
            )
        else:
            assert completion == "JEP330.java a/ d".split()

    @pytest.mark.complete("java -cp '' ", cwd="java")
    def test_5(self, completion):
        assert completion == "JEP330.java a/".split()

    @pytest.mark.complete("java -jar ", cwd="java")
    def test_6(self, completion):
        assert completion == "a/ bashcomp.jar bashcomp.war".split()

    @pytest.mark.complete("javadoc -sourcepath java/a:java/a/c ")
    def test_sourcepath_1(self, completion):
        """sourcepath should be split by `:`"""
        assert completion == "c"

    @pytest.mark.complete("javadoc -sourcepath java/?:java/x ")
    def test_sourcepath_2(self, completion):
        """pathname expansion should not happen after splitting the argument by
        `:`"""
        assert not completion

    @pytest.mark.complete("javadoc -sourcepath java/a ")
    def test_packages_1(self, completion):
        assert completion == "c"

    @pytest.mark.complete("javadoc -sourcepath java/a x")
    def test_packages_2(self, completion):
        assert not completion

    @pytest.mark.complete(
        "javadoc -sourcepath java/a x", shopt=dict(failglob=True)
    )
    def test_packages_3(self, completion):
        assert not completion

    @pytest.mark.complete("javadoc -sourcepath java/a ", env=dict(IFS="a"))
    def test_packages_4(self, completion):
        assert completion == "c"

    def test_packages_5(self, bash):
        """_comp_cmd_java__packages should not modify the outerscope `cur`"""
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable("cur", "a.b.c")
            assert_bash_exec(
                bash,
                "_comp_test_f() { local cword=3 words=(javadoc -sourcepath java/a a.b.c); COMPREPLY+=(); _comp_cmd_java__packages; }; _comp_test_f",
            )

    @pytest.mark.complete("javadoc -sourcepath java a.")
    def test_packages_6(self, completion):
        """A period in package names should not be converted to slash."""
        assert completion == "c"



import pytest


class TestXz:
    @pytest.mark.complete("xz ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("xz -d xz/")
    def test_2(self, completion):
        assert (
            completion
            == "a/ bashcomp.lzma bashcomp.tar.xz bashcomp.tlz "
            "bashcomp.xz".split()
        )

    @pytest.mark.complete("xz xz/")
    def test_3(self, completion):
        assert completion == "a/ bashcomp.tar".split()

    @pytest.mark.complete("xz ~")
    def test_4(self, completion):
        assert completion

    @pytest.mark.complete("xz -", require_cmd=True)
    def test_5(self, completion):
        assert completion



import pytest


class TestChflags:
    @pytest.mark.complete("chflags no")
    def test_basic(self, completion):
        assert completion

    @pytest.mark.complete("chflags -")
    def test_options(self, completion):
        assert completion and "-P" not in completion

    @pytest.mark.complete("chflags -R -")
    def test_options_after_R(self, completion):
        assert "-P" in completion

    @pytest.mark.complete("chflags -v sappend ")
    def test_first_word(self, completion):
        assert completion



import pytest


class TestJsonXs:
    @pytest.mark.complete("json_xs ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("json_xs -")
    def test_2(self, completion):
        assert completion



import pytest

from conftest import assert_complete, create_dummy_filedirs


@pytest.mark.bashcomp(temp_cwd=True)
class TestKdvi:
    def test_1(self, bash):
        files, dirs = create_dummy_filedirs(
            (
                ".dvi .DVI .dvi.bz2 .DVI.bz2 .dvi.gz .DVI.gz .dvi.Z .DVI.Z "
                ".txt"
            ).split(),
            "foo".split(),
        )

        completion = assert_complete(bash, "kdvi ")
        assert completion == [
            x
            for x in sorted(files + ["%s/" % d for d in dirs])
            if x.lower() != ".txt"
        ]



import os
import shutil
import sys
import tempfile
from pathlib import Path

import pytest

from conftest import assert_bash_exec, assert_complete, bash_env_saved


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+COMPREPLY=")
class TestUnitCompgenFiledir:
    @pytest.fixture(scope="class")
    def functions(self, request, bash):
        assert_bash_exec(
            bash,
            "_f() { local cur;_comp_get_words cur; unset -v COMPREPLY; _comp_compgen_filedir; }; "
            "complete -F _f f; "
            "complete -F _f -o filenames f2",
        )
        assert_bash_exec(
            bash,
            "_g() { local cur;_comp_get_words cur; unset -v COMPREPLY; _comp_compgen_filedir e1; }; "
            "complete -F _g g",
        )
        assert_bash_exec(
            bash,
            "_fd() { local cur;_comp_get_words cur; unset -v COMPREPLY; _comp_compgen_filedir -d; };"
            "complete -F _fd fd",
        )

    @pytest.fixture(scope="class")
    def non_windows_testdir(self, request, bash):
        if sys.platform.startswith("win"):
            pytest.skip("Filenames not allowed on Windows")
        tempdir = Path(tempfile.mkdtemp(prefix="bash-completion_filedir"))
        request.addfinalizer(lambda: shutil.rmtree(str(tempdir)))
        subdir = tempdir / 'a"b'
        subdir.mkdir()
        (subdir / "d").touch()
        subdir = tempdir / "a*b"
        subdir.mkdir()
        (subdir / "j").touch()
        subdir = tempdir / r"a\b"
        subdir.mkdir()
        (subdir / "g").touch()
        return tempdir

    @pytest.fixture(scope="class")
    def utf8_ctype(self, bash):
        # TODO: this likely is not the right thing to do. Instead we should
        # grab the setting from the running shell, possibly eval $(locale)
        # in a subshell and grab LC_CTYPE from there. That doesn't seem to work
        # either everywhere though.
        lc_ctype = os.environ.get("LC_CTYPE", "")
        if "UTF-8" not in lc_ctype:
            pytest.skip("Applicable only in LC_CTYPE=UTF-8 setups")
        return lc_ctype

    def test_1(self, bash):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable("cur", "")
            assert_bash_exec(bash, 'cur="" _comp_compgen_filedir >/dev/null')

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_2(self, bash, functions, funcname):
        completion = assert_complete(bash, "%s ab/" % funcname, cwd="_filedir")
        assert completion == "e"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_3(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s a\ b/" % funcname, cwd="_filedir"
        )
        assert completion == "i"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_4(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s a\'b/" % funcname, cwd="_filedir"
        )
        assert completion == "c"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_5(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s a\&b/" % funcname, cwd="_filedir"
        )
        assert completion == "f"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_6(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s a\$" % funcname, cwd="_filedir"
        )
        assert completion == "b/"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_7(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s 'ab/" % funcname, cwd="_filedir"
        )
        assert completion == "e'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_8(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s 'a b/" % funcname, cwd="_filedir"
        )
        assert completion == "i'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_9(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s 'a$b/" % funcname, cwd="_filedir"
        )
        assert completion == "h'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_10(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s 'a&b/" % funcname, cwd="_filedir"
        )
        assert completion == "f'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_11(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r'%s "ab/' % funcname, cwd="_filedir"
        )
        assert completion == 'e"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_12(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r'%s "a b/' % funcname, cwd="_filedir"
        )
        assert completion == 'i"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_13(self, bash, functions, funcname):
        completion = assert_complete(
            bash, "%s \"a'b/" % funcname, cwd="_filedir"
        )
        assert completion == 'c"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_14(self, bash, functions, funcname):
        completion = assert_complete(
            bash, '%s "a&b/' % funcname, cwd="_filedir"
        )
        assert completion == 'f"'

    @pytest.mark.complete(r"fd a\ ", cwd="_filedir")
    def test_15(self, functions, completion):
        assert completion == "b/"

    @pytest.mark.complete("g ", cwd="_filedir/ext")
    def test_16(self, functions, completion):
        assert completion == sorted("ee.e1 foo/ gg.e1 ii.E1".split())

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_17(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s a\$b/" % funcname, cwd="_filedir"
        )
        assert completion == "h"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_18(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r"%s \[x" % funcname, cwd="_filedir/brackets"
        )
        assert completion == r"\]"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_19(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, '%s a\\"b/' % funcname, cwd=non_windows_testdir
        )
        assert completion == "d"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_20(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, r"%s a\\b/" % funcname, cwd=non_windows_testdir
        )
        assert completion == "g"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_21(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, "%s 'a\"b/" % funcname, cwd=non_windows_testdir
        )
        assert completion == "d'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_22(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, rf"{funcname} '{non_windows_testdir}/a\b/"
        )
        assert completion == "g'"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_23(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, r'%s "a\"b/' % funcname, cwd=non_windows_testdir
        )
        assert completion == 'd"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_24(self, bash, functions, funcname, non_windows_testdir):
        completion = assert_complete(
            bash, r'%s "a\\b/' % funcname, cwd=non_windows_testdir
        )
        assert completion == 'g"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_25(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r'%s "a\b/' % funcname, cwd="_filedir"
        )
        assert completion == '\b\b\bb/e"'

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_26(self, bash, functions, funcname):
        completion = assert_complete(
            bash, r'%s "a\$b/' % funcname, cwd="_filedir"
        )
        assert completion == 'h"'

    @pytest.mark.xfail(reason="TODO: non-ASCII issues with test suite?")
    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_27(self, bash, functions, funcname, utf8_ctype):
        completion = assert_complete(bash, "%s aé/" % funcname, cwd="_filedir")
        assert completion == "g"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_28_dot_1(self, bash, functions, funcname):
        """Exclude . and .. when the completion is attempted for '.[TAB]'"""
        completion = assert_complete(bash, r"%s ." % funcname, cwd="_filedir")
        assert completion == [".dotfile1", ".dotfile2"]

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_28_dot_2(self, bash, functions, funcname):
        """Exclude . and .. when the completion is attempted for 'dir/.[TAB]'"""
        completion = assert_complete(bash, r"%s _filedir/." % funcname)
        assert completion == [".dotfile1", ".dotfile2"]

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_28_dot_3(self, bash, functions, funcname):
        """Include . when the completion is attempted for '..[TAB]'"""
        completion = assert_complete(bash, r"%s .." % funcname, cwd="_filedir")
        assert completion == "/"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_28_dot_4(self, bash, functions, funcname):
        """Include . when the completion is attempted for '..[TAB]'"""
        completion = assert_complete(bash, r"%s _filedir/.." % funcname)
        assert completion == "/"

    @pytest.mark.parametrize("funcname", "f f2".split())
    def test_29_dotdot(self, bash, functions, funcname):
        """Complete files starting with "..".

        These types of files are used by the go kubernetes atomic writer [0],
        and presumably other types of systems, and we want to make sure they
        will be completed correctly.

        [0] https://pkg.go.dev/k8s.io/kubernetes/pkg/volume/util#AtomicWriter.Write
        """
        completion = assert_complete(
            bash, r"%s .." % funcname, cwd="_filedir/dotdot/"
        )
        assert completion == [
            "../",
            "..2016_02_01_15_04_05.123456",
            "..data",
            "..folder/",
        ]



import pytest


class TestArp:
    @pytest.mark.complete(
        "arp ", require_cmd=True, skipif='test ! "$(arp 2>/dev/null)"'
    )
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("arp -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestUseradd:
    @pytest.mark.complete("useradd ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("useradd -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("useradd -R shells --shell=")
    def test_chroot_shells(self, completion):
        assert completion == "/bash/completion/canary"



import pytest


class TestVgcfgbackup:
    @pytest.mark.complete(
        "vgcfgbackup -",
        require_cmd=True,
        xfail="! vgcfgbackup --help &>/dev/null",
    )
    def test_1(self, completion):
        assert completion



from itertools import chain

import pytest

from conftest import assert_bash_exec, assert_complete, bash_env_saved

LIVE_HOST = "bash_completion"


class TestScp:
    @pytest.mark.complete("scp -F config ", cwd="scp")
    def test_basic(self, hosts, completion):
        expected = sorted(
            chain(
                (
                    "%s:" % x
                    for x in chain(
                        hosts,
                        # From fixtures/scp/config
                        "gee hut".split(),
                        # From fixtures/scp/known_hosts
                        "blah doo ike".split(),
                    )
                ),
                # Local filenames
                ["bin/", "config", "known_hosts", r"spaced\ \ conf"],
            )
        )
        assert completion == expected

    @pytest.mark.complete("scp -F 'spaced  conf' ", cwd="scp")
    def test_basic_spaced_conf(self, hosts, completion):
        expected = sorted(
            chain(
                (
                    "%s:" % x
                    for x in chain(
                        hosts,
                        # From "fixtures/scp/spaced  conf"
                        "gee jar".split(),
                        # From fixtures/scp/known_hosts
                        "blah doo ike".split(),
                    )
                ),
                # Local filenames
                ["bin/", "config", "known_hosts", r"spaced\ \ conf"],
            )
        )
        assert completion == expected

    @pytest.mark.complete("scp -F")
    def test_capital_f_without_space(self, completion):
        assert completion
        assert not any(
            "option requires an argument -- F" in x for x in completion
        )

    @pytest.fixture(scope="class")
    def live_pwd(self, bash):
        try:
            return assert_bash_exec(
                bash,
                "ssh -o 'Batchmode yes' -o 'ConnectTimeout 1' "
                "%s pwd 2>/dev/null" % LIVE_HOST,
                want_output=True,
            ).strip()
        except AssertionError:
            pytest.skip("Live host %s not available" % LIVE_HOST)

    @pytest.mark.complete("scp %s:" % LIVE_HOST, sleep_after_tab=2)
    def test_live(self, live_pwd, completion):
        """
        To support this test, configure a HostName entry for LIVE_HOST
        in ssh's configs, e.g. ~/.ssh/config or /etc/ssh/ssh_config.

        Connection to it must open sufficiently quickly for the
        ConnectTimeout and sleep_after_tab settings.
        """
        assert completion == f"{LIVE_HOST}:{live_pwd}/"

    @pytest.mark.complete("scp -o Foo=")
    def test_option_arg(self, completion):
        assert not completion  # and no errors either

    @pytest.mark.complete(
        "scp hostname-not-expected-to-exist-in-known-hosts:",
        shopt=dict(nullglob=True),
    )
    def test_remote_path_with_nullglob(self, completion):
        assert not completion

    @pytest.mark.complete(
        "scp hostname-not-expected-to-exist-in-known-hosts:",
        shopt=dict(failglob=True),
    )
    def test_remote_path_with_failglob(self, completion):
        assert not completion

    def test_remote_path_with_spaces(self, bash):
        assert_bash_exec(bash, "ssh() { echo 'spaces in filename.txt'; }")
        completion = assert_complete(bash, "scp remote_host:spaces")
        assert_bash_exec(bash, "unset -f ssh")
        assert completion == r"\\\ in\\\ filename.txt"

    def test_xfunc_remote_files(self, bash):
        with bash_env_saved(bash) as bash_env:
            bash_env.save_variable("COMPREPLY")
            bash_env.write_variable(
                "PATH",
                "$PWD/scp/bin:$PATH",
                quote=False,
            )
            bash_env.write_variable("cur", "local:shared/default/")
            completions_regular_escape = (
                assert_bash_exec(
                    bash,
                    r'_comp_compgen -x scp remote_files; printf "%s\n" "${COMPREPLY[@]}"',
                    want_output=True,
                )
                .strip()
                .splitlines()
            )
            completions_less_escape = (
                assert_bash_exec(
                    bash,
                    r'_comp_compgen -x scp remote_files -l; printf "%s\n" "${COMPREPLY[@]}"',
                    want_output=True,
                )
                .strip()
                .splitlines()
            )
        assert completions_regular_escape == [
            "shared/default/bar ",
            r"shared/default/bar\\\ bar.d/",
            "shared/default/foo ",
            "shared/default/foo.d/",
        ]
        assert completions_less_escape == [
            "shared/default/bar ",
            r"shared/default/bar\ bar.d/",
            "shared/default/foo ",
            "shared/default/foo.d/",
        ]



import subprocess

import pytest


class TestLook:
    @pytest.mark.complete("look foo")
    def test_1(self, completion):
        try:
            subprocess.check_call(
                "look foo 2>/dev/null | command grep -q ^foo", shell=True
            )
        except BaseException:
            assert not completion
        else:
            assert completion



import pytest


class TestFile:
    @pytest.mark.complete("file ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("file -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import os

import pytest

from conftest import assert_bash_exec, bash_env_saved, prepare_fixture_dir


@pytest.mark.bashcomp(cmd=None, cwd="_comp_load")
class TestCompLoad:
    @pytest.fixture
    def fixture_dir(self, request, bash):
        """Construct the fixture directory in a temporary directory.

        Some of the tests use specific setups of symbolic links.  However, if
        we put the symbolic links in the static fixture directory, Automake
        resolves them for tarballs.  As a result, the tests fail when the files
        are extracted from the tarballs.  There does not seem to be any option
        to change the behavior of Automake.

        We instead manually set up all symbolic links needed for the tests
        here.  The other normal files and directories are statically included
        in the repository as "/test/fixtures/_comp_load".  We first copy the
        statically included files and directories to a temporary directory and
        set up symbolic links.
        """

        tmpdir, _, _ = prepare_fixture_dir(request, files=[], dirs=[])
        assert_bash_exec(bash, "cp -R %s/* %s/" % (os.getcwd(), tmpdir))
        assert_bash_exec(bash, "mkdir -p %s/bin" % tmpdir)
        assert_bash_exec(
            bash, "ln -sf ../prefix1/bin/cmd1 %s/bin/cmd1" % tmpdir
        )
        assert_bash_exec(
            bash, "ln -sf ../prefix1/sbin/cmd2 %s/bin/cmd2" % tmpdir
        )
        return str(tmpdir)

    def test_userdir_1(self, bash, fixture_dir):
        with bash_env_saved(bash) as bash_env:
            bash_env.chdir(fixture_dir)
            bash_env.write_variable(
                "BASH_COMPLETION_USER_DIR",
                "$PWD/userdir1:$PWD/userdir2:$BASH_COMPLETION_USER_DIR",
                quote=False,
            )
            bash_env.write_variable(
                "PATH", "$PWD/prefix1/bin:$PWD/prefix1/sbin", quote=False
            )
            output = assert_bash_exec(
                bash, "_comp_load cmd1", want_output=True
            )
            assert output.strip() == "cmd1: sourced from userdir1"
            output = assert_bash_exec(
                bash, "_comp_load cmd2", want_output=True
            )
            assert output.strip() == "cmd2: sourced from userdir2"

    def test_PATH_1(self, bash, fixture_dir):
        with bash_env_saved(bash) as bash_env:
            bash_env.chdir(fixture_dir)
            bash_env.write_variable(
                "PATH", "$PWD/prefix1/bin:$PWD/prefix1/sbin", quote=False
            )
            output = assert_bash_exec(
                bash, "_comp_load cmd1", want_output=True
            )
            assert output.strip() == "cmd1: sourced from prefix1"
            output = assert_bash_exec(
                bash, "_comp_load cmd2", want_output=True
            )
            assert output.strip() == "cmd2: sourced from prefix1"
            output = assert_bash_exec(
                bash, "complete -p cmd2", want_output=True
            )
            assert " cmd2" in output
            output = assert_bash_exec(
                bash, 'complete -p "$PWD/prefix1/sbin/cmd2"', want_output=True
            )
            assert "/prefix1/sbin/cmd2" in output

    def test_cmd_path_1(self, bash, fixture_dir):
        with bash_env_saved(bash) as bash_env:
            bash_env.chdir(fixture_dir)
            assert_bash_exec(bash, "complete -r cmd1 || :", want_output=None)
            output = assert_bash_exec(
                bash, "_comp_load prefix1/bin/cmd1", want_output=True
            )
            assert output.strip() == "cmd1: sourced from prefix1"
            output = assert_bash_exec(
                bash, 'complete -p "$PWD/prefix1/bin/cmd1"', want_output=True
            )
            assert "/prefix1/bin/cmd1" in output
            assert_bash_exec(bash, "! complete -p cmd1", want_output=None)
            output = assert_bash_exec(
                bash, "_comp_load prefix1/sbin/cmd2", want_output=True
            )
            assert output.strip() == "cmd2: sourced from prefix1"
            output = assert_bash_exec(
                bash, "_comp_load bin/cmd1", want_output=True
            )
            assert output.strip() == "cmd1: sourced from prefix1"
            output = assert_bash_exec(
                bash, "_comp_load bin/cmd2", want_output=True
            )
            assert output.strip() == "cmd2: sourced from prefix1"

    def test_cmd_path_2(self, bash, fixture_dir):
        with bash_env_saved(bash) as bash_env:
            bash_env.chdir(fixture_dir)
            bash_env.write_variable("PATH", "$PWD/bin:$PATH", quote=False)
            output = assert_bash_exec(
                bash, "_comp_load cmd1", want_output=True
            )
            assert output.strip() == "cmd1: sourced from prefix1"
            output = assert_bash_exec(
                bash, "_comp_load cmd2", want_output=True
            )
            assert output.strip() == "cmd2: sourced from prefix1"

    def test_cmd_intree_precedence(self, bash, fixture_dir):
        """
        Test in-tree, i.e. completions/$cmd relative to the main script
        has precedence over location derived from PATH.
        """
        with bash_env_saved(bash) as bash_env:
            bash_env.chdir(fixture_dir)
            bash_env.write_variable("PATH", "$PWD/prefix1/bin", quote=False)
            # The in-tree `sh` completion should be loaded here,
            # and cause no output, unlike our `$PWD/prefix1/bin/sh` canary.
            assert_bash_exec(bash, "_comp_load sh", want_output=False)

    def test_option_like_cmd_name(self, bash):
        assert_bash_exec(bash, "! _comp_load -- --non-existent")



import pytest


class TestComplete:
    @pytest.mark.complete("complete -")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(r"\complete -")
    def test_2(self, completion):
        assert completion



import pytest


class TestPython:
    @pytest.mark.complete("python ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("python -", require_cmd=True)
    def test_2(self, completion):
        assert len(completion) > 1

    @pytest.mark.complete("python -c ")
    def test_3(self, completion):
        assert not completion

    @pytest.mark.complete("python shared/default/")
    def test_4(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]

    @pytest.mark.complete("python -c foo shared/default/")
    def test_5(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete("python -c foo -")
    def test_6(self, completion):
        assert not completion

    @pytest.mark.complete("python -m foo -")
    def test_7(self, completion):
        assert not completion

    @pytest.mark.complete("python -m sy", require_cmd=True)
    def test_8(self, completion):
        assert completion

    @pytest.mark.complete("python -m json.", require_cmd=True)
    def test_9(self, completion):
        assert "json.tool" in completion

    @pytest.mark.complete(
        "python -b",
        require_cmd=True,
        skipif="! python -h | command grep -qwF -- -bb",
    )
    def test_bb(self, completion):
        assert "-bb" in completion

    @pytest.mark.complete("python foo ", cwd="python")
    def test_script_arg(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -- foo ", cwd="python")
    def test_script_arg_with_double_hyphen(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -m foo bar -p ", cwd="python")
    def test_module_arg(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python foo bar -p ", cwd="python")
    def test_script_arg_after_option(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -- foo bar -p ", cwd="python")
    def test_script_arg_after_option_with_double_hyphen(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -m foo bar -p ", cwd="python")
    def test_module_arg_after_option(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -mfoo bar -p ", cwd="python")
    def test_module_arg_after_option_with_connected_m_arg(self, completion):
        assert "bar.txt" in completion

    @pytest.mark.complete("python -- ", cwd="python")
    def test_script_name(self, completion):
        assert "bar.txt" not in completion

    @pytest.mark.complete("python -W -mfoo ", cwd="python")
    def test_script_name_with_fake_m_arg(self, completion):
        """In this case, -mfoo looks like an option to specify the module, but
        it should not be treated as the module name because it is an option
        argument to -W."""
        assert "bar.txt" not in completion



import pytest


class TestChmod:
    # No completion here until mode completion is implemented
    @pytest.mark.complete("chmod ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("chmod 755 ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("chmod -", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("chmod -x ")
    def test_4(self, completion):
        assert completion

    @pytest.mark.complete("chmod -77 ")
    def test_5(self, completion):
        assert completion



import pytest


class TestMkisofs:
    @pytest.mark.complete("mkisofs ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mkisofs -uid ")
    def test_2(self, completion):
        assert not [x for x in completion if not x.isdigit()]

    @pytest.mark.complete("mkisofs -gid ")
    def test_3(self, completion):
        assert not [x for x in completion if not x.isdigit()]



import pytest


class TestPrintenv:
    @pytest.mark.complete("printenv ")
    def test_empty(self, completion):
        assert completion

    @pytest.mark.complete("printenv PAT")
    def test_path(self, completion):
        assert completion == "H" or "PATH" in completion

    @pytest.mark.complete(
        "printenv -",
        require_cmd=True,
        xfail="! printenv --help 2>&1 | command grep -q '^[[:space:]]*-'",
    )
    def test_options(self, completion):
        assert completion



import pytest


class TestKtutil:
    @pytest.mark.complete("ktutil ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ktutil -")
    def test_2(self, completion):
        assert completion



import pytest


class TestKillall:
    # "p": Assume our process name completion runs ps and at least it is shown
    @pytest.mark.complete("killall p")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("killall --signal ")
    def test_2(self, completion):
        assert all(x in completion for x in "INT KILL TERM".split())

    @pytest.mark.complete("killall ")
    def test_3(self, completion):
        assert "command=" not in completion

    @pytest.mark.complete("killall -", require_cmd=True)
    def test_4(self, completion):
        assert completion



import pytest


class TestKcov:
    @pytest.mark.complete("kcov ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("kcov --exclude-patter", require_cmd=True)
    def test_2(self, completion):
        assert completion == "n="
        assert completion.endswith("=")

    @pytest.mark.complete("kcov -l 42,")
    def test_3(self, completion):
        assert completion



import pytest


class TestWsimport:
    @pytest.mark.complete("wsimport ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "wsimport -",
        require_cmd=True,
        xfail=(
            "! (wsimport -help 2>&1 || :) | "
            "command grep -q -- '[[:space:]]-'"
        ),
    )
    def test_2(self, completion):
        assert completion



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(ignore_env=r"^[+-]PORTSDIR=", temp_cwd=True)
class TestPortinstall:
    @pytest.fixture(scope="class")
    def portsdir(self, request, bash):
        assert_bash_exec(bash, "PORTSDIR=$PWD")
        assert_bash_exec(
            bash,
            "command sed -e s,PORTSDIR,$PORTSDIR,g "
            '"$SRCDIRABS"/fixtures/pkgtools/ports/INDEX.dist >INDEX',
        )
        assert_bash_exec(bash, "cp INDEX INDEX-5")

    @pytest.mark.complete("portinstall ", env=dict(PORTSDIR="$PWD"))
    def test_1(self, completion, portsdir):
        assert (
            completion
            == "bash-2.05b.007_6 bash-3.1.17 bash-completion-20060301_2 "
            "shells/bash shells/bash-completion shells/bash2".split()
        )



import pytest


@pytest.mark.bashcomp(cmd="dpkg-deb")
class TestDpkgDeb:
    @pytest.mark.complete("dpkg-deb --c", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("dpkg-deb --show b", cwd="dpkg")
    def test_show(self, completion):
        assert completion == "ash-completion-test-subject.deb"



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(cmd=None)
class TestUnitGetFirstArg:
    @pytest.fixture(scope="class")
    def functions(self, bash):
        assert_bash_exec(
            bash,
            '_comp__test_unit() { local -a "words=$1"; local cword=$2 REPLY=; shift 2; _comp_get_first_arg "$@" && printf "%s\\n" "$REPLY"; return 0; }',
        )

    def _test(self, bash, words, cword, args=""):
        return assert_bash_exec(
            bash,
            '_comp__test_unit "%s" %d %s' % (words, cword, args),
            want_output=None,
        ).strip()

    def test_1(self, bash, functions):
        assert_bash_exec(bash, "_comp__test_unit '()' 0")

    def test_2(self, bash, functions):
        output = self._test(bash, "(a b)", 2)
        assert output == "b"

    def test_3(self, bash, functions):
        output = self._test(bash, "(a bc)", 2)
        assert output == "bc"

    def test_4(self, bash, functions):
        output = self._test(bash, "(a b c)", 2)
        assert output == "b"

    def test_5(self, bash, functions):
        """Neither of the current word and the command name should be picked
        as the first argument"""
        output = self._test(bash, "(a b c)", 1)
        assert output == ""

    def test_6(self, bash, functions):
        """Options starting with - should not be picked as the first
        argument"""
        output = self._test(bash, "(a -b -c d e)", 4)
        assert output == "d"

    def test_7_single_hyphen(self, bash, functions):
        """- should be counted as an argument representing stdout/stdin"""
        output = self._test(bash, "(a -b - c -d e)", 5)
        assert output == "-"

    def test_8_double_hyphen_1(self, bash, functions):
        """any word after -- should be picked"""
        output = self._test(bash, "(a -b -- -c -d e)", 5)
        assert output == "-c"

    def test_8_double_hyphen_2(self, bash, functions):
        """any word after -- should be picked only without any preceding argument"""
        output = self._test(bash, "(a b -- -c -d e)", 5)
        assert output == "b"

    def test_9_skip_optarg_1(self, bash, functions):
        output = self._test(bash, "(a -b -c d e f)", 5, '-a "@(-c|--foo)"')
        assert output == "e"

    def test_9_skip_optarg_2(self, bash, functions):
        output = self._test(bash, "(a -b --foo d e f)", 5, '-a "@(-c|--foo)"')
        assert output == "e"

    def test_9_skip_optarg_3(self, bash):
        output = self._test(bash, "(a -b - c d e)", 5, '-a "-b"')
        assert output == "c"

    def test_9_skip_optarg_4(self, bash):
        output = self._test(bash, "(a -b -c d e f)", 5, '-a "-[bc]"')
        assert output == "d"

    def test_9_skip_optarg_5(self, bash):
        output = self._test(bash, "(a +o b c d)", 4, '-a "+o"')
        assert output == "c"

    def test_9_skip_optarg_6(self, bash):
        output = self._test(bash, "(a -o -o -o -o b c)", 6, '-a "-o"')
        assert output == "b"

    def test_9_skip_optarg_7(self, bash):
        output = self._test(bash, "(a -o -- -b -c d e)", 6, '-a "-o"')
        assert output == "d"



import pytest


@pytest.mark.bashcomp(cmd="gnome-mplayer", ignore_env=r"^[+-]XDG_DATA_DIRS=")
class TestGnomeMplayer:
    @pytest.mark.complete("gnome-mplayer ")
    def test_1(self, completion):
        assert completion

    # XDG_DATA_DIRS set to a dir with no schemas results in
    # "GLib-GIO-ERROR **: No GSettings schemas are installed on the system"
    # and a core dump on --help on Ubuntu 14.
    @pytest.mark.complete(
        "gnome-mplayer -", require_cmd=True, pre_cmds=("unset XDG_DATA_DIRS",)
    )
    def test_2(self, completion):
        assert completion



import pytest


class TestIpv6calc:
    @pytest.mark.complete("ipv6calc -", require_cmd=True)
    def test_1(self, completion):
        assert "--action" in completion

    @pytest.mark.complete("ipv6calc --in ", require_cmd=True)
    def test_2(self, completion):
        assert completion



import os
from tempfile import mkstemp

import pytest

from conftest import assert_complete, is_bash_type


@pytest.mark.bashcomp(cmd="slapt-src")
class TestSlaptSrc:
    @pytest.fixture(scope="class")
    def slapt_srcrc(self, request, bash):
        fd, fname = mkstemp(prefix="slapt-srcrc.", text=True)
        request.addfinalizer(lambda: os.remove(fname))
        with os.fdopen(fd, "w") as f:
            print(
                "BUILDDIR=%s/"
                % os.path.join(
                    bash.cwd, *"slackware usr src slapt-src".split()
                ),
                file=f,
            )
        return fname

    @pytest.mark.complete("slapt-src -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("slapt-src --bu", require_cmd=True)
    def test_2(self, completion):
        assert completion == "ild" or "--build" in completion

    @pytest.mark.complete("slapt-src --ins", require_cmd=True)
    def test_3(self, completion):
        assert completion == "tall" or "--install" in completion

    def test_install(self, bash, slapt_srcrc):
        if not is_bash_type(bash, "slapt-src"):
            pytest.skip("slapt-src not found")
        completion = assert_complete(
            bash, "slapt-src --config %s --install " % slapt_srcrc
        )
        assert completion == "abc:4 qwe:2.1".split()



import pytest


class TestPkgadd:
    # require_cmd is not strictly true here, but...
    @pytest.mark.complete("pkgadd ", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest


class TestIconv:
    @pytest.mark.complete(
        "iconv -", require_cmd=True, xfail="! iconv --help &>/dev/null"
    )
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("iconv -f UTF", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("iconv ")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("iconv -f ")
    def test_4(self, completion):
        assert "..." not in completion



import pytest


class TestQemu:
    @pytest.mark.complete("qemu ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("qemu -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("qemu -k ", require_cmd=True)
    def test_keymaps(self, completion):
        assert any(x.lower().startswith("en") for x in completion)



import pytest


class TestDd:
    @pytest.mark.complete("dd --", require_longopt=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("dd bs")
    def test_2(self, completion):
        assert completion == "="



import pytest


@pytest.mark.bashcomp(pre_cmds=("HOME=$PWD/mplayer",))
class TestMplayer:
    @pytest.mark.complete("mplayer ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mplayer -h", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestChkconfig:
    @pytest.mark.complete("chkconfig -")
    def test_1(self, completion):
        assert completion

    # systemd may not be running e.g. in a docker container, and listing
    # services will then fail.
    @pytest.mark.complete(
        "chkconfig ", xfail="! systemctl list-units &>/dev/null"
    )
    def test_2(self, completion):
        assert completion



import pytest


class TestLd:
    @pytest.mark.complete("ld ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ld -", require_longopt=True)
    def test_options(self, completion):
        assert completion



import pytest


class TestXev:
    @pytest.mark.complete("xev ")
    def test_basic(self, completion):
        assert not completion

    @pytest.mark.complete("xev -", require_cmd=True)
    def test_options(self, completion):
        assert completion



"""Async function pytest completion fixture."""


async def test_positive():
    pass


async def non_test_negative():
    pass


class Testing:
    async def test_positive(self):
        pass

    async def non_test_negative(self):
        pass



import pytest


class TestOpenssl:
    @pytest.mark.complete("openssl ", require_cmd=True)
    def test_1(self, completion):
        assert completion
        assert all(
            x in completion for x in "md5 x509 aes-128-cbc dgst pkey".split()
        )

    @pytest.mark.complete("openssl pkey -cipher ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("openssl dgst -s", require_cmd=True)
    def test_3(self, completion):
        assert completion
        assert any(x.startswith("-sha") for x in completion)



import pytest


class TestCarton:
    @pytest.mark.complete("carton ", require_cmd=True)
    def test_commands(self, completion):
        assert all(x in completion for x in "help install".split())

    @pytest.mark.complete("carton install -", require_cmd=True)
    def test_install_options(self, completion):
        assert all(x in completion for x in "--cached --help".split())



import shlex

import pytest

from conftest import TestUnitBase, assert_bash_exec


@pytest.mark.bashcomp(cmd=None)
class TestUnitLooksLikePath(TestUnitBase):
    @pytest.mark.parametrize(
        "thing_looks_like",
        (
            ("", False),
            ("foo", False),
            ("/foo", True),
            ("foo/", True),
            ("foo/bar", True),
            (".", True),
            ("../", True),
            ("~", True),
            ("~foo", True),
        ),
    )
    def test_1(self, bash, thing_looks_like):
        thing, looks_like = thing_looks_like
        output = assert_bash_exec(
            bash,
            f"_comp_looks_like_path {shlex.quote(thing)}; printf %s $?",
            want_output=True,
            want_newline=False,
        )
        is_zero = output.strip() == "0"
        assert (looks_like and is_zero) or (not looks_like and not is_zero)



import pytest


@pytest.mark.bashcomp(cmd="locale-gen")
class TestLocaleGen:
    # require_cmd is not strictly true here, but...
    @pytest.mark.complete("locale-gen ", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("locale-gen --", require_longopt=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestLzma:
    @pytest.mark.complete("lzma ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lzma -")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("lzma -d xz/")
    def test_3(self, completion):
        assert completion == "a/ bashcomp.lzma bashcomp.tlz".split()

    @pytest.mark.complete("lzma ~")
    def test_4(self, completion):
        assert completion



import pytest


class TestMypy:
    @pytest.mark.complete("mypy ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mypy --", require_longopt=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("mypy --non-existent-option=--")
    def test_3(self, completion):
        assert not completion



import pytest


class TestGm:
    @pytest.mark.complete("gm ", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("gm help ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("gm time ", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("gm version ")
    def test_4(self, completion):
        assert not completion



import pytest


@pytest.mark.bashcomp(pre_cmds=("export RI='-d ri'",))
class TestRi:
    @pytest.mark.complete("ri -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ri --dump=ri/")
    def test_2(self, completion):
        assert completion == "BashCompletion/ cache.ri".split()

    @pytest.mark.complete("ri BashCompletio", require_cmd=True)
    def test_3(self, completion):
        assert completion == "n"
